/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

// Zeus imports
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.InvalidCollectionTypeException;
import org.enhydra.zeus.util.ClassUtils;
import org.enhydra.zeus.util.NamingUtils;

/**
 * <p>
 *  <code>{@link Binding}</code> represents an arbitrary Java construct,
 *    and provides a representation-independent means of showing
 *    how a constraint (presumably from an XML Schema, DTD, Relax
 *    schema, etc.) maps to Java. It provides a layer of representation
 *    between the constraint representation and generated Java.
 * </p><p>
 *  This implementation of <code>Binding</code> deals with
 *    the basic functionality of all bindings. It leaves specific
 *    binding implementations to deal with only their functionality.
 * </p>
 *
 * @author Brett McLaughlin
 * @author Sean Ogle
 */
public abstract class BaseBinding {
    
    /** The XML name of the binding */
    protected String xmlName;

    /** The XML namespace URI associated with the XML name of the binding */
    protected String xmlNamespaceURI;

    /** The XML type of the binding */
    protected String xmlType;

    /** The XML namespace URI associated with the XML type of the binding */
    protected String xmlTypeNamespaceURI;
    
    /** The XML parent type for this binding */
    protected String xmlParentType;
    
    /** The XML namespace URI for this binding's XML parent type */
    protected String xmlParentTypeNamespaceURI;
    
    /** Whether this is the XML root element */
    protected boolean isXMLRootElement;

    /** The Java name of the binding */
    protected String javaName;
    
    /** The Java type of the binding */
    protected String javaType;

    /** The Java name of the binding that is a legal Java identifier. */
    protected String javaVariableName;

    /** The Java interface package of the binding */
    protected String javaInterfacePackage;
    
    /** The Java implementation package of the binding */
    protected String javaImplementationPackage;
    
    /** The Java Collection class for this binding */
    protected String javaCollectionClass;
    
    /** Whether this binding is Java-serializable */
    protected boolean isJavaSerializable;
    
    /**
     * <p>
     *  Default constructor.
     * </p>
     */
    public BaseBinding() {
        isJavaSerializable = true;
        isXMLRootElement = false;
        xmlNamespaceURI = "";
        xmlTypeNamespaceURI = "";
        xmlParentTypeNamespaceURI = "";
    }

    /**
     * <p>
     *  This will return the XML name of the binding. This should
     *    be the actual XML name of the binding as would be seen in an XML
     *    document.
     * </p>
     *
     * @return <code>String</code> - the XML name of the binding.
     */
    public String getXMLName() {
        return xmlName;
    }
    /**
     * <p>
     *  This will set the XML name of the binding.
     * </p>
     *
     * @param xmlName the XML name of the binding.
     */
    public void setXMLName(String xmlName) {
        if (xmlName == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null XML name.");
        }
        
        this.xmlName = xmlName;
    }
    
    /**
     * <p>
     *  The will return the XML namespace URI associated with the XML name of
     *    this binding. A value of the empty string indicates that there is no 
     *    URI associated with the XML name of this binding.
     * </p>
     *
     * @return <code>String</code> - the XML namespace URI associated with the 
     *         XML name of this binding, or an empty string if no namespace URI.
     */
    public String getXMLNamespaceURI() {
        return xmlNamespaceURI;
    }

    /**
     * <p>
     *  The will set the XML namespace URI associated with the XML name of
     *    this binding. A value of the empty string indicates that there is no 
     *    URI associated with the XML name of this binding.
     * </p>
     *
     * @param xmlNamespaceURI the XML namespace URI
     */
    public void setXMLNamespaceURI(String xmlNamespaceURI) {
        if (xmlNamespaceURI == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null XML namespace URI. To specify no namespace URI, " +
                "use the empty String (\"\")");
        }
        
        this.xmlNamespaceURI = xmlNamespaceURI;
    }
    
    /**
     * <p>
     *  This will return the XML local name associated with the XML type of this
     *    binding. When using a DTD, the XML type is the same as the XML name.
     *    However, when using an XML Schema, the type may be different from the
     *    name.
     * </p>
     *
     * @return <code>String</code> - the XML local name of the XML type of this 
     *         binding.
     */
    public String getXMLType() {
        return xmlType;
    }

    /**
     * <p>
     *  This will set the XML local name associated with the XML type of this
     *    binding.
     * </p>
     *
     * @param xmlType the XML type.
     */
    public void setXMLType(String xmlType) {
        if (xmlType == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null XML type.");
        }
        
        this.xmlType = xmlType;
    }
    
    /**
     * <p>
     *  The will return the XML namespace URI associated with the XML type of
     *    this binding. A value of the empty string indicates that there is no 
     *    URI associated with the XML type of this binding.
     * </p><p>
     *  While this is not
     *    useful in DTD binding, it allows types like <code>xsd:string</code> to
     *    be distinguished from something like <code>myPrefix:string</code>, 
     *    which a user might define.
     * </p>
     *
     * @return <code>String</code> - the XML namespace URI associated with the 
     *         XML type of this binding.
     */
    public String getXMLTypeNamespaceURI() {
        return xmlTypeNamespaceURI;
    }

    /**
     * <p>
     *  This will set the XML namespace URI associated with the XML type of
     *   this binding. A value of the empty string indicates that there is no 
     *   URI associated with the XML type of this binding.
     * </p>
     *
     * @param xmlTypeNamespaceURI the XML type's namespace URI.
     */
    public void setXMLTypeNamespaceURI(String xmlTypeNamespaceURI) {
        if (xmlTypeNamespaceURI == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null XML type namespace URI. To specify no namespace URI, " +
                "use the empty String (\"\")");
        }
        
        this.xmlTypeNamespaceURI = xmlTypeNamespaceURI;
    }
    
    /**
     * <p>
     *  This will return the XML local name associated with the XML type of this
     *    binding's XML parent. When using a DTD, the XML type is the same as 
     *    the XML name.  However, when using an XML Schema, the type may be 
     *    different from the name.
     * </p><p>
     *  If there is no parent XML type, such as for the root element of a
     *    document, this value is <code>null</code>.
     *
     * @return <code>String</code> - the XML local name of the XML type of this 
     *         binding's parent.
     */
    public String getXMLParentType() {
        return xmlParentType;
    }

    /**
     * <p>
     *  This will set the XML local name associated with the XML type of this
     *    binding's XML parent.
     * </p>
     *
     * @param xmlParentType the XML type for this binding's parent.
     */
    public void setXMLParentType(String xmlParentType) {
        if (xmlParentType == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null parent XML type.");
        }
        
        this.xmlParentType = xmlParentType;
    }
    
    /**
     * <p>
     *  The will return the XML namespace URI associated with the XML type of
     *    this binding's XML parent. A value of the empty string indicates that
     *    there is no URI associated with the XML type of this binding's parent.
     * </p><p>
     *  While this is not
     *    useful in DTD binding, it allows types like <code>xsd:string</code> to
     *    be distinguished from something like <code>myPrefix:string</code>, 
     *    which a user might define.
     * </p>
     *
     * @return <code>String</code> - the XML namespace URI associated with the 
     *         XML type of this binding's parent.
     */
    public String getXMLParentTypeNamespaceURI() {
        return xmlParentTypeNamespaceURI;
    }

    /**
     * <p>
     *  This will set the XML namespace URI associated with the XML type of
     *   this binding's XML parent. A value of the empty string indicates that 
     *   there is no URI associated with the XML type of this binding's parent.
     * </p>
     *
     * @param xmlParentTypeNamespaceURI the XML namespace URI of the 
     *        parent type.
     */
    public void setXMLParentTypeNamespaceURI(String xmlParentTypeNamespaceURI) {
        if (xmlParentTypeNamespaceURI == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null parent XML type namespace URI. To specify no namespace " +
                "URI, use the empty String (\"\")");
        }
        
        this.xmlParentTypeNamespaceURI = xmlParentTypeNamespaceURI;
    }
    
    /**
     * <p>
     *  This sets whether this binding represents an XML element that is
     *    the root element of the document.
     * </p>
     *
     * @param isXMLRootElement whether this is the root element.
     */
    public void setIsXMLRootElement(boolean isXMLRootElement) {
        this.isXMLRootElement = isXMLRootElement;
    }
    
    /**
     * <p>
     *  This indicates if this binding represents an XML element that is the
     *    root element of the document.
     * </p>
     *
     * @return <code>boolean</code> - whether this is the root element.
     */
    public boolean isXMLRootElement() {
        return isXMLRootElement;
    }

    /**
     * <p>
     *  This will return the Java name of the binding. This should
     *    be the valid Java name of the binding, which is most
     *    often a variable name. This is the string used to build the methods
     *    <code>setXXX()</code> and <code>getXXX()</code>. Note that this is 
     *    not necessarily the variable name used by the generators, but they 
     *    will provide the <code>setXXX()</code> and <code>getXXX()</code>
     *    based on this name.
     * </p>
     *
     * @return <code>String</code> - the Java name of the binding.
     */
    public String getJavaName() {
        return javaName;
    }

    /**
     * <p>
     *  The will set the name of the Java variable associated with this binding.
     * </p>
     *
     * @param javaName the new variable name.
     */
    public void setJavaName(String javaName) {
        if (javaName == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null Java name.");
        }
        
        this.javaName = javaName;
    }
    
    /**
     * <p>
     *  This will return the Java type associated with the
     *    binding. This is not a fully qualified name for the type. To get the
     *    fully qualified name for the type prepend the value returned from
     *    getJavaPackage() to the value returned by this method (with a "."
     *    inserted in between if the package is not the empty string).
     * </p>
     *
     * @return <code>String</code> - the Java type of the binding.
     */
    public String getJavaType() {
        return javaType;
    }
    
    /**
     * <p>
     *  This will set the Java type for this binding.
     * </p>
     *
     * @param javaType the Java type for this binding.
     */
    public void setJavaType(String javaType) {
        if (javaType == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null Java type.");
        }
        
        this.javaType = javaType;
    }

    /**
     * <p>
     *  This will return the Java name that can be used for a variable name in
     *    generated code. It is guaranteed to be a legal Java identifier.
     * </p><p>
     *   To illustrate the difference between the Java name and its variable 
     *   name, consider this simple example:
     * <pre><code>
     *   <!ELEMENT default EMPTY>
     * </code></pre>
     *   With the above DTD the XML name is "default". However, "default" 
     *   is a Java keyword, but <code>setDefault()</code> and 
     *   <code>getDefault()</code> are not illegal. So for a typical name 
     *   binding, the Java name would be "default" and the variable name 
     *   would (could) be "safeDefault". The generated interface would still 
     *   contain <code>setDefault()</code> and <code>getDefault()</code>.
     * </p><p>
     *  Note that it is not necessary for the variable name to always (or even
     *    often) be different from the normal Java name.
     * </p>
     *
     * @return <code>String</code> - the Java variable name.
     */
    public String getJavaVariableName() {
        return javaVariableName;
    }

    /**
     * <p>
     *  This will set the Java name that can be used for a variable name in
     *    generated code. This must be a legal Java identifier, and the first
     *    character must not be an upper case letter.
     * </p>
     *
     * @param javaVariableName the name to use for the Java member variable.
     */
    public void setJavaVariableName(String javaVariableName) {
        if (javaVariableName == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null Java variable name.");
        }
        
        this.javaVariableName = javaVariableName;
    }

    /**
     * <p>
     *  This will return the Java package name for this binding's interface
     *    class.
     * </p>
     *
     * @return <code>String</code> - the Java interface package associated with 
     *         the binding's Java type.
     */
    public String getJavaInterfacePackage() {
        return javaInterfacePackage;
    }

    /**
     * <p>
     *  This will set the Java package for this class's interface.
     * </p>
     *
     * @param javaInterfacePackage the name of the interface package for this
     *        binding.
     */
    public void setJavaInterfacePackage(String javaInterfacePackage) {
        if (javaInterfacePackage == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null Java interface package name.");
        }
        
        this.javaInterfacePackage = javaInterfacePackage;
    }
    
    /**
     * <p>
     *  This will return the Java package name for this binding's implementation
     *    class.
     * </p>
     *
     * @return <code>String</code> - the Java implementation package associated 
     *         with the binding's Java type.
     */
    public String getJavaImplementationPackage() {
        return javaImplementationPackage;
    }

    /**
     * <p>
     *  This will set the Java package for this class's implementation.
     * </p>
     *
     * @param javaImplementationPackage the name of the implementation package 
     *        for this binding.
     */
    public void setJavaImplementationPackage(String javaImplementationPackage) {
        if (javaImplementationPackage == null) {
            throw new IllegalArgumentException("A Binding cannot have a " +
                "null implementation package name.");
        }
        
        this.javaImplementationPackage = javaImplementationPackage;
    }
    
    /**
     * <p>
     *  This will set the Java collection class to use for this binding,
     *    if it is a collection. This will be a fully-qualified Java class,
     *    such as <code>java.util.List</code>.
     * </p>
     *
     * @param javaCollectionClass the collection class for this binding.
     * @throws <code>InvalidCollectionTypeException</code> - when the supplied
     *         collection class is not a valid collection type.
     */
    public void setJavaCollectionClass(String javaCollectionClass)
        throws InvalidCollectionTypeException {
        
        if (javaCollectionClass == null) {
            throw new IllegalArgumentException("A Binding cannot have a null " +
                "Java collection class.");
        }
        if (!ClassUtils.isCollectionClass(javaCollectionClass)) {
            throw new InvalidCollectionTypeException(javaCollectionClass);
        }
        
        this.javaCollectionClass = javaCollectionClass;
    }
    
    /**
     * <p>
     *  This will return the Java collection class to use for this binding,
     *    if it is a collection. This will be a fully qualified Java class,
     *    such as <code>java.util.List</code>.
     * </p>
     *
     * @return <code>String</code> - the Java collection class for this binding.
     */
    public String getJavaCollectionClass() {
        return javaCollectionClass;
    }
    
    /**
     * <p>
     *  This will set whether this binding should be a Java-serializable class.
     * </p>
     *
     * @param isJavaSerializable whether this binding should be Java
     *        serializable (true) or not (false).
     */
    public void setIsJavaSerializable(boolean isJavaSerializable) {
        this.isJavaSerializable = isJavaSerializable;
    }
    
    /**
     * <p>
     *  This will indicate whether this binding is Java-serializable (indicated
     *    by a <code>boolean</code> value of "true"), or not (indicated by a
     *    <code>boolean</code> value of "false").
     * </p>
     *
     * @return <code>boolean</code> - whether this binding is Java-serializable.
     */
    public boolean isJavaSerializable() {
        return isJavaSerializable;
    }
}
