/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.util;

import java.util.HashMap;
import java.util.Map;

// Zeus imports
import org.enhydra.zeus.ZeusDefaults;

/**
 * <p>
 *  <code>NamingUtils</code> is a Zeus utility class that provides methods
 *    which deal with the naming issues involved in data binding.  Specifically,
 *    this class will deal with deriving Java class names, deriving Java 
 *    variable names, deriving XML element names, and validating legal Java 
 *    identifiers.
 * <p>
 *
 * @author Brett McLaughlin
 * @author Robert Sese
 * @author Maciej Zawadzki
 */
public class NamingUtils {

    /** Prefix to append to names that clash with Java reserved words. */
    public static final String XML_PREFIX = "xml";

    /** The default alternative representation for illegal characters. */
    public static final String DEFAULT_MAPPING = "";

    /**
     * A map of illegal characters mapped to legal alternative representations.
     */
    private static Map illegalCharMap = new HashMap();

    /** A map of all the Java reserved words. */
    private static HashMap reservedWords = new HashMap();

    /** An array of all the Java reserved words to be stored in a map. */ 
    private static String reservedWordsArray[] = { "abstract",
        "boolean", "break", "byte", "case", "catch", "char", "class",
        "char", "const", "continue", "default", "do", "double", "else",
        "extends", "false", "final", "finally", "float", "for",
        "goto", "if", "implements", "import", "instanceof", "int",
        "interface", "long", "native", "new", "null", "package",
        "private", "protected", "public", "return", "short", "static",
        "super", "switch", "synchronized", "this", "throw", "throws",
        "transient", "true", "try", "void", "volatile", "while" };

    // Initialize the reserved words map and the illegal characters map.
    static {
        for (int i = 0; i < reservedWordsArray.length; i++) {
            reservedWords.put(reservedWordsArray[i], new Integer(i));
        }
        reservedWordsArray = null;

        // XXX: Add more illegal chars mappings...we don't have to check too
        //   many because we can assume XML naming rules, though
        illegalCharMap.put(new Character('.'), DEFAULT_MAPPING);
        illegalCharMap.put(new Character('-'), DEFAULT_MAPPING);
        illegalCharMap.put(new Character(':'), DEFAULT_MAPPING);
    }

    /**
     * <p>
     *  Prevent direct instantiation.
     * </p>
     */
    private NamingUtils() {}
    
    /**
     * <p>
     *  This will indicate whether the supplied name is a legal Java class
     *    name.
     * </p>
     *
     * @param className the Java class name to check
     * @return <code>boolean</code> - whether the supplied class is a legal
     *         Java class name.
     */
    public static boolean isLegalJavaClassName(String className) {
        if (className == null) {
            throw new IllegalArgumentException("A non-null String must be " +
                "supplied to NamingUtils methods.");
        }
        
        for (int i=0, len=className.length(); i<len; i++) {
            if ((i == 0) &&
                (!Character.isJavaIdentifierStart(className.charAt(i)))) {
                return false;
            } else if (!Character.isJavaIdentifierPart(className.charAt(i))) {
                return false;
            }
        }
        
        // If we got here, all is OK
        return true;
    }
    
    /**
     * <p>
     *  This will indicate whether the supplied name is a legal Java package
     *    name.
     * </p>
     *
     * @param packageName the Java class name to check
     * @return <code>boolean</code> - whether the supplied package is a legal
     *         Java package name.
     */
    public static boolean isLegalJavaPackageName(String packageName) {
        if (packageName == null) {
            throw new IllegalArgumentException("A non-null String must be " +
                "supplied to NamingUtils methods.");
        }
        
        for (int i=0, len=packageName.length(); i<len; i++) {
            if ((i == 0) &&
                (!Character.isJavaIdentifierStart(packageName.charAt(i)))) {
                return false;
            } else if (
                (!Character.isJavaIdentifierPart(packageName.charAt(i))) &&
                (!(packageName.charAt(i) == '.'))) {

                return false;
            }
        }
        
        // If we got here, all is OK
        return true;
    }

    /**
     * <p>
     *  Returns the value that the given key is mapped to.  This value will
     *   replace any occurence of the key when a name contains illegal 
     *   characters for a Java identifier.
     * </p>
     *
     * @param key the key mapped to the value to return.
     * @return <code>String</code> - the value the given key is mapped to.
     */
    public static String getCharMapping(Character key) {
        if (key == null) {
            throw new IllegalArgumentException("A non-null Character must be " +
                "supplied to NamingUtils methods.");
        }
        
        return (String)illegalCharMap.get(key);
    }

    /**
     * <p>
     *  Maps the given key to the given value.  This value will replace any
     *    occurence of the key when  a name contains illegal characters
     *    for a Java identifier.
     * </p>
     *
     * @param key the key to map the given value to.
     * @param value the value to map the given key to.
     */
    public static void setCharMapping(Character key, String value) {
        if (key == null) {
            throw new IllegalArgumentException("A non-null char key must be " +
                "supplied to NamingUtils methods.");
        }
        if (value == null) {
            throw new IllegalArgumentException("A non-null value must be " +
                "supplied to NamingUtils methods.");
        }
        
        illegalCharMap.put(key, value);
    }
    
    /**
     * <p>
     *  Returns a conventional Java name from an XML name.  Any illegal 
     *    characters are replaced. Collision with reserved words is <i>not</i>
     *    checked, as this is not used directly in class generation.
     * </p><p>
     *  For example, calling <code>getJavaName("my-element")</code> would 
     *    return "myElement" as the Java class name.
     * </p>
     *
     * @param xmlName the XML name to convert to a Java name.
     * @return <code>String</code> - the converted Java name.
     */
    public static String getJavaName(String xmlName) {
        if (xmlName == null) {
            throw new IllegalArgumentException("A non-null XML name must be " +
                "supplied to NamingUtils methods.");
        }
        
        // Handle the PCDATA special case
        if (xmlName.equals(ZeusDefaults.PCDATA_XML_NAME)) {
            return convertIllegalChars(ZeusDefaults.PCDATA_JAVA_NAME);
        }
        
        return convertReservedWord(convertIllegalChars(xmlName));
    }

    /**
     * <p>
     *  Returns the conventional Java variable name from an XML name.  The
     *    initial character is uncapitalized, and any illegal characters are
     *    replaced.  If there is a collision with a Java reserved word, the name
     *    is prefixed with <code>{@link #XML_PREFIX}</code>.
     * </p><p>
     *  For example, calling <code>getJavaVariableName("XmlName")</code>
     *    would return "xmlName" as the Java variable name.
     * </p>
     *
     * @param xmlName the XML name to convert to a variable name.
     * @return <code>String</code> - the converted Java variable name.
     */
    public static String getJavaVariableName(String xmlName) {
        if (xmlName == null) {
            throw new IllegalArgumentException("A non-null XML name must " +
                "be supplied to NamingUtils methods.");
        }
        
        // Handle the PCDATA special case
        if (xmlName.equals(ZeusDefaults.PCDATA_XML_NAME)) {
            return convertReservedWord(
                CapitalizationUtils.initialLower(
                    ZeusDefaults.PCDATA_JAVA_NAME));
        }
        
        return convertReservedWord(
            convertIllegalChars(CapitalizationUtils.initialLower(xmlName)));
    }

    /**
     * <p>
     *  Returns the Java variable name from a Java class name assuming that the
     *    variable will be a collection based variable (i.e. ending with "List" 
     *    as in "myVariableList").  The initial character is uncapitalized, and 
     *    any illegal characters are replaced.  There is no check for a 
     *    collision with a Java reserved word because the assumption is that 
     *    the variable has some word appended to it that identifies the fact 
     *    that the variable denotes a collection.
     * </p><p>
     *  For example, calling 
     *    <code>getJavaCollectionVariableName("MyClass")</code> would return 
     *    "myClass" as the Java variable name.
     * </p>
     *
     * @param className the class name to convert to a collection based
     *        variable name.
     * @return <code>String</code> - the converted Java variable name.
     */
    public static String getJavaCollectionVariableName(String className) {
        if (className == null) {
            throw new IllegalArgumentException("A non-null class name must " +
                "be supplied to NamingUtils methods.");
        }
        
        return convertReservedWord(
            convertIllegalChars(CapitalizationUtils.initialLower(className)));
    }
    
    /**
     * <p>
     *  Returns the conventional Java class name from an XML name.  The initial
     *    character is capitalized, and any illegal characters are replaced.
     *    There is no check for a collision with a Java reserved word because 
     *    all Java reserved words have their initial character lower case.
     * </p><p>
     *  For example, calling <code>getJavaClassName("my-element")</code> would 
     *    return "MyElement" as the Java class name.
     * </p>
     *
     * @param xmlName the XML name to convert to a Java class name.
     * @return <code>String</code> - the converted Java name.
     */
    public static String getJavaClassName(String xmlName) {
        if (xmlName == null) {
            throw new IllegalArgumentException("A non-null XML name must be " +
                "supplied to NamingUtils methods.");
        }
        
        return convertReservedWord(
            convertIllegalChars(CapitalizationUtils.initialUpper(xmlName)));
    }
    
    /**
     * <p>
     *  This will handle conversion from an XML type to a Java type. It first
     *    attempts to perform an XML Schema type to Java type conversion. For
     *    example, if the XML type is "string", it would convert this to
     *    the Java type "String". If that is unsuccessful, it returns the 
     *    type unchanged, assuming that it is a proper Java class name.
     * </p>
     *
     * @param xmlType the XML type to convert to a Java type.
     * @return <code>String</code> - the converted Java type.
     */
    public static String getJavaType(String xmlType) {
        String javaType;
        try {
            javaType = SchemaUtils.getJavaType(xmlType);
            return javaType;
        } catch (UnsupportedSchemaTypeException e) {
            // ignored
        }
        
        // Ensure a legal Java class name is returned
        return getJavaClassName(xmlType);
    }
    
    /**
     * <p>
     *  Returns the XML name from a typical accessor method.  The naming
     *    convention for such a method is "get&lt;variable-name&gt;".  The first
     *    three characters are discarded, and the initial character is
     *    uncapitalized.
     * </p><p>
     *  For example, calling 
     *    <code>getXMLElementNameFromAccessor("getMyVariable")</code> would 
     *    return "myVariable" as the XML name.  This method would also work 
     *    with typical mutator methods.
     * </p>
     *
     * @param accessor the name of the accessor method to derive the XML name
     *        from.
     * @return <code>String</code> - the converted XML name.
     */
    public static String getXMLElementNameFromAccessor(String accessor) {
        if (accessor == null) {
            throw new IllegalArgumentException("A non-null accessor must " +
                "be supplied to NamingUtils methods.");
        }
        
        return CapitalizationUtils.initialLower(accessor.substring(3));
    }
    
    /**
     * <p>
     *  This will take a fully qualified Java class, and return just the name
     *    of the class, without any package qualifier. For example, invoking
     *    <code>removePackage("java.util.List");</code> would return
     *    "List".
     * </p>
     *
     * @param className the Java class name to remove the package from.
     * @return <code>String</code> - the Java class, without package.
     */
    public static String removePackage(String className) {
        String returnValue = className;
        
        if (returnValue.indexOf(".") > -1) {
            returnValue = returnValue.substring(
                returnValue.lastIndexOf(".") + 1);
        }
        
        return returnValue;
    }

    /**
     * <p>
     *  Checks whether the given name collides with a Java reserved word.  If
     *    there is a collision, the name is returned prefixed with
     *    <code>{@link #XML_PREFIX}</code>, else it is returned unchanged.
     * </p> 
     *    
     * @param name the name to check.
     * @return <code>String</code> - the converted name if there is a collision
     *         with a Java reserved word, else the same name.
     */
    private static String convertReservedWord(String name) {
        if (name == null) {
            throw new IllegalArgumentException("A non-null name must " +
                "be supplied to NamingUtils methods.");
        }
        
        if (reservedWords.get(name) == null) {
            return name;
        } else {
            return new StringBuffer().append(XML_PREFIX)
                .append(name).toString();
        }
    }

    /**
     * <p>
     *  Checks whether each character in the given name is valid for a Java
     *    identifier.  If there are illegal characters, the name is returned 
     *    with each illegal character replaced with its value in the internal 
     *    map of illegal <code>Character</code>s to replacement 
     *    <code>String</code>s, else the same name is returned.  If the 
     *    illegal character can be interpreted as a separator character 
     *    (such as the '-' in my-attribute), then the character following the 
     *    separator character will be capitalized (i.e. my-attribute becomes 
     *    myAttribute).
     * </p> 
     *    
     * @param name the name to check.
     * @return <code>String</code> - the converted name if illegal characters 
     *         are present, else the same name.
     */
    private static String convertIllegalChars(String name) {
        if (name == null) {
            throw new IllegalArgumentException("A non-null name must " +
                "be supplied to NamingUtils methods.");
        }
        
        StringBuffer verifiedName = new StringBuffer();
        char[] characters = name.toCharArray();

        // XXX: Uppercase the next letter after all illegal chars, or just the
        // ones that can be interpreted as separator chars?
        boolean upperNext = false;

        for (int i = 0; i < characters.length; i++) {
            char c = characters[i];

            if (i == 0) {
                if (Character.isJavaIdentifierStart(c) == false) {
                    String legalValue =
                        (String) illegalCharMap.get(new Character(c));

                    if (legalValue == null) {

                        // XXX: Throw an exception or append DEFAULT_MAPPING?
                        throw new IllegalArgumentException("No mapping for " +
                            "illegal character: " + c);
                    } else {
                        verifiedName.append(legalValue);

                        // XXX Does it make sense to uppercase the next char
                        // if the illegal char is the first char of the
                        // identifier?
                        upperNext = true;
                    }
                } else {
                    verifiedName.append(c);
                }
            } else {
                if (Character.isJavaIdentifierPart(c) == false) {
                    String legalValue =
                        (String) illegalCharMap.get(new Character(c));

                    if (legalValue == null) {

                        // XXX: Throw an exception or append DEFAULT_MAPPING?
                        throw new IllegalArgumentException("No mapping for " +
                            "illegal character: " + c);
                    } else {
                        verifiedName.append(legalValue);
                        upperNext = true;
                    }
                } else {
                    if (upperNext) {
                        verifiedName.append(Character.toUpperCase(c));
                        upperNext = false;
                    } else {
                        verifiedName.append(c);
                    }
                }
            }
        }
        return verifiedName.toString();
    }
}
