/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.enhydra.zeus.AbstractBindingTest;
import org.enhydra.zeus.Binding;

/**
 * <p>
 *  <code>AbstractContainerTest</code> is a base test for 
 *    <code>{@link Container}</code> objects.
 * </p><p>
 *  Concrete <code>Container</code> test cases should extend this class, 
 *    override the <code>{@link #createContainer}</code> method, and add any 
 *    tests specific to the class being tested.
 * </p>
 *
 * @author Robert Sese
 * @author Brett McLaughlin
 */
public abstract class AbstractContainerTest extends AbstractBindingTest {

    /**
     * <p>
     *  This creates a new <code>{@link Container}</code> object that will be 
     *    used to test <code>Container</code>-level methods.
     * </p>
     *
     * @return <code>Container</code> - Container object to test with.
     */
    protected abstract Container createContainer();

    /**
     * <p>
     *  This constructs a new <code>AbstractContainerTest</code>.
     * </p>
     *
     * @param name the name of the test case.
     */
    public AbstractContainerTest(String name) {
        super(name);
    }
    
    /**
     * <p>
     *  This method maeks it easier to call these
     *    tests dynamically.
     * </p>
     *
     * @return <code>TestSuite</code> - corresponds to this 
     *         <code>TestCase</code>.
     */
    public static Test suite() {
        return new TestSuite(AbstractContainerTest.class);
    }

    /**
     * <p>
     *  This creates a new <code>Binding</code> object that will be used to 
     *    test <code>Binding</code> methods.
     * </p>
     *
     * @return <code>Binding</code> - the created <code>Binding</code>
     */
    public Binding createBinding() {

        // Just delegate to the Container subclass.
        return createContainer();
    }

    /**
     * <p>
     *  This tests the <code>{@link Container#addProperty}</code> method.
     * </p>
     */ 
    public void testAddProperty() {
        Container container = createContainer();
        Property property = new AtomicProperty("xmlName", "xmlType");
        container.addProperty(property);

        /**
         * XXX: Do we need to define equals for Bindings, or is comparsion by
         * name enough?
         */
        List properties = container.getProperties();
        boolean hasProperty = false;
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property p = (Property)i.next();
            if (p.getXMLName().equals(property.getXMLName())) {
                hasProperty = true;
                break;
            }
        }
        assertTrue(hasProperty);
    }

    /**
     * <p>
     *  This tests the <code>{@link Container#removeProperty}</code> method.
     * </p>
     */ 
    public void testRemoveProperty() {
        Container container = createContainer();
        Property property = new AtomicProperty("xmlName", "xmlType");
        property.setJavaName("javaName");
        container.addProperty(property);
        container.removeProperty(property.getJavaName());

        /*
         * XXX: Do we need to define equals for Bindings, or is comparsion by
         * name enough?
         */
        List properties = container.getProperties();
        
        boolean removedProperty = true;
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property p = (Property)i.next();
            if (p.getXMLName().equals(property.getXMLName())) {
                removedProperty = false;
                break;
            }
        }
        assertTrue(removedProperty);
    }

    /**
     * <p>
     *  This tests the <code>{@link Container#getProperties}</code> method.
     * </p>
     */ 
    public void testGetProperties() {
        Container container = createContainer();
        Property property1 = new AtomicProperty("xmlName1", "xmlType1");
        Property property2 = new AtomicProperty("xmlName2", "xmlType2");
        container.addProperty(property1);
        container.addProperty(property2);

        List properties = container.getProperties();
        boolean hasProperty1 = false;
        boolean hasProperty2 = false;
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            Property p = (Property)i.next();
            if (p.getXMLName().equals(property1.getXMLName())) {
                hasProperty1 = true;
            } else if (p.getXMLName().equals(property2.getXMLName())) {
                hasProperty2 = true;
            }
        }
        assertTrue(hasProperty1 && hasProperty2);
    }
    
    /**
     *  This tests the <code>{@link Container#setProperties}</code> method.
     */ 
    public void testSetProperties() {
        Container container = createContainer();
        Property property1 = new AtomicProperty("xmlName1", "xmlType1");
        Property property2 = new AtomicProperty("xmlName2", "xmlType2");
        List properties = new LinkedList();
        properties.add(property1);
        properties.add(property2);
        container.setProperties(properties);
        
        List containerProperties = container.getProperties();
        boolean hasProperty1 = false;
        boolean hasProperty2 = false;
        boolean hasOtherProperties = false;
        for (Iterator i = containerProperties.iterator(); i.hasNext(); ) {
            Property property = (Property)i.next();
            if (property.getXMLName().equals(property1.getXMLName())) {
                hasProperty1 = true;
            } else if (property.getXMLName().equals(property2.getXMLName())) {
                hasProperty2 = true;
            } else {
                hasOtherProperties = true;
            }
        }
        assertTrue(hasProperty1 && hasProperty2 && !hasOtherProperties);
    }

    /**
     *  This tests the <code>{@link Container#clearProperties}</code> method.
     */ 
    public void testClearProperties() {
        Container container = createContainer();
        Property property1 = new AtomicProperty("xmlName1", "xmlType1");
        Property property2 = new AtomicProperty("xmlName2", "xmlType2");
        container.addProperty(property1);
        container.addProperty(property2);
        container.clearProperties();
        
        List properties = container.getProperties();
        assertTrue(properties.size() == 0);
    }
}
