/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

import java.util.BitSet;
import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.enhydra.zeus.AbstractBindingTest;
import org.enhydra.zeus.Binding;

/**
 * <p>
 *  <code>AbstractPropertyTest</code> is a base test for 
 *    <code>{@link Property}</code> objects.
 * </p><p>
 *  Concrete <code>Property</code> test cases should extend this class, 
 *    override the <code>{@link #createProperty}</code> method, and add any 
 *    tests specific to the class being tested.
 * </p>
 *
 * @author Robert Sese
 * @author Brett McLaughlin
 */
public abstract class AbstractPropertyTest extends AbstractBindingTest {

    /**
     * <p>
     *  This creates a new <code>{@link Property}</code> object that will be 
     *    used to test <code>Property</code>-level methods.
     * </p>
     *
     * @return <code>Property</code> - Property object to test with.
     */
    protected abstract Property createProperty();

    /**
     * <p>
     *  This constructs a new <code>AbstractPropertyTest</code>.
     * </p>
     *
     * @param name the name of the test case.
     */
    public AbstractPropertyTest(String name) {
        super(name);
    }
    
    /**
     * <p>
     *  This method maeks it easier to call these
     *    tests dynamically.
     * </p>
     *
     * @return <code>TestSuite</code> - corresponds to this 
     *         <code>TestCase</code>.
     */
    public static Test suite() {
        return new TestSuite(AbstractPropertyTest.class);
    }
    
    /**
     * <p>
     *  This creates a new <code>Binding</code> object that will be used to 
     *    test <code>Binding</code> methods.
     * </p>
     *
     * @return <code>Binding</code> - the created <code>Binding</code>
     */
    public Binding createBinding() {

        // Delegate to the Property being tested.
        return createProperty();
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#setModifier}</code> method.
     * </p>
     */
    public void testSetModifier() {
        Property property = createProperty();
        BitSet modifiers = new BitSet();
        modifiers.set(Property.ACCESS_PRIVATE);
        modifiers.set(Property.SOURCE_ATTLIST);

        property.setModifier(modifiers);
        assertEquals(modifiers, property.getModifier());

        modifiers = new BitSet();
        modifiers.set(Property.ACCESS_PUBLIC);
        modifiers.set(Property.SOURCE_ELEMENT);

        property.setModifier(modifiers);
        assertEquals(modifiers, property.getModifier());
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#getModifier}</code> method.
     * </p>
     */
    public void testGetModifier() {
        Property property = createProperty();
        BitSet modifiers = new BitSet();
        modifiers.set(Property.ACCESS_PRIVATE);
        modifiers.set(Property.SOURCE_ATTLIST);

        property.setModifier(modifiers);
        assertEquals(modifiers, property.getModifier());
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#getModifierString}</code> method.
     * </p>
     */
    public void testGetModifierString() {
        Property property = createProperty();
        BitSet modifiers = new BitSet();
        modifiers.set(Property.ACCESS_PRIVATE);
        modifiers.set(Property.SOURCE_ATTLIST);
        String expected = "private";

        property.setModifier(modifiers);
        assertEquals(expected, property.getModifierString());
        
        modifiers = new BitSet();
        modifiers.set(Property.ACCESS_PUBLIC);
        modifiers.set(Property.STORAGE_STATIC);
        modifiers.set(Property.MUTABILITY_FINAL);
        expected = "public static final";

        property.setModifier(modifiers);
        assertEquals(expected, property.getModifierString());
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#setIsCollection}</code> method.
     * </p>
     */
    public void testSetIsCollection() {
        Property property = createProperty();
        boolean expected = true;
        property.setIsCollection(expected);
        assertEquals(expected, property.isCollection());

        expected = false;
        property.setIsCollection(expected);
        assertEquals(expected, property.isCollection());
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#isCollection}</code> method.
     * </p>
     */
    public void testIsCollection() {
        Property property = createProperty();
        boolean expected = true;
        property.setIsCollection(expected);
        assertEquals(expected, property.isCollection());
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#hasDefaultValue}</code> method.
     * </p>
     */
    public void testHasDefaultValue() {
        Property property = createProperty();
        boolean expected = true;
        property.setDefaultValue("defaultValue");
        assertEquals(expected, property.hasDefaultValue());
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#setDefaultValue}</code> method.
     * </p>
     */
    public void testSetDefaultValue() {
        Property property = createProperty();
        String expected1 = "defaultValue";
        property.setDefaultValue(expected1);
        assertEquals(expected1, (String)property.getDefaultValue());

        Integer expected2 = new Integer(10);
        property.setDefaultValue(expected2);
        assertEquals(expected2, (Integer)property.getDefaultValue());
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#getDefaultValue}</code> method.
     * </p>
     */
    public void testGetDefaultValue() {
        Property property = createProperty();
        String expected1 = "defaultValue";
        property.setDefaultValue(expected1);
        assertEquals(expected1, (String)property.getDefaultValue());
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#hasEnumeration}</code> method.
     * </p>
     */
    public void testHasEnumeration() {
        Property property = createProperty();
        boolean expected = true;
        property.setEnumeration(new Vector());
        assertEquals(expected, property.hasEnumeration());
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#setEnumeration}</code> method.
     * </p>
     */
    public void testSetEnumeration() {
        Property property = createProperty();
        Vector expected1 = new Vector();
        expected1.add("value1");
        expected1.add("value2");
        property.setEnumeration(expected1);
        assertEquals(expected1, property.getEnumeration());

        Vector expected2 = new Vector();
        expected2.add(new Integer(1));
        expected2.add(new Integer(2));
        property.setEnumeration(expected2);
        assertEquals(expected2, property.getEnumeration());
    }

    /**
     * <p>
     *  This tests the <code>{@link Property#getEnumeration}</code> method.
     * </p>
     */
    public void testGetEnumeration() {
        Property property = createProperty();
        Vector expected1 = new Vector();
        expected1.add("value1");
        expected1.add("value2");
        property.setEnumeration(expected1);
        assertEquals(expected1, property.getEnumeration());
    }
}
