/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

import java.util.BitSet;
import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * <p>
 *  This is a test case for the <code>{@link AtomicProperty}</code> class.
 * </p>
 *
 * @author Robert Sese
 * @author Brett McLaughlin
 */
public class AtomicPropertyTest extends AbstractPropertyTest {

    /** Enumerated values for the test. */
    private Vector enumeratedValues;

    /**
     * The modifiers (i.e. public, volatile, etc) for the test AtomicProperty.
     */
    private BitSet modifiers;

    /** The test <code>AtomicProperty.</code> */
    private AtomicProperty atomicProperty;

    /**
     * <p>
     *  This creates a new <code>{@link Property}</code> object that will be 
     *    used to test <code>Property</code>-level methods.
     * </p>
     *
     * @return <code>Property</code> - Property object to test with.
     */
    protected Property createProperty() {
        return new AtomicProperty("xmlName", "xmlType");
    }

    /**
     * <p>
     *  This constructs a new <code>AbstractContainerTest</code>.
     * </p>
     *
     * @param name the name of the test case.
     */
    public AtomicPropertyTest(String name) {
        super(name);
    }

    /**
     * <p>
     *  This method maeks it easier to call these
     *    tests dynamically.
     * </p>
     *
     * @return <code>TestSuite</code> - corresponds to this 
     *         <code>TestCase</code>.
     */
    public static Test suite() {
        return new TestSuite(AtomicPropertyTest.class);
    }

    /**
     * <p>
     *  This establishes the test fixture.
     * </p>
     */
    protected void setup() {
        enumeratedValues = new Vector();
        enumeratedValues.add("value1");
        enumeratedValues.add("value2");

        modifiers = new BitSet();
        modifiers.set(Property.ACCESS_PRIVATE);
        modifiers.set(Property.SOURCE_ATTLIST);

        atomicProperty = new AtomicProperty("xmlName", "xmlType");
    }

    /**
     * <p>
     *  This releases any resources in use.
     * </p>
     */
    protected void teardown() {
        enumeratedValues = null;
        modifiers = null;
        atomicProperty = null;
    }

    /**
     * <p>
     *  This tests a valid construction.
     * </p>
     */
    public void testConstructor1() {
        setup();
        atomicProperty = new AtomicProperty("xmlName",
                                            "namespaceURI",
                                            "xmlType",
                                            "typeNamespaceURI",
                                            modifiers,
                                            enumeratedValues,
                                            "defaultValue");
        assertTrue(atomicProperty != null);
        assertTrue(atomicProperty.getXMLName().equals("xmlName"));
        assertTrue(atomicProperty.getXMLNamespaceURI().equals("namespaceURI"));
        assertTrue(atomicProperty.getXMLType().equals("xmlType"));
        assertTrue(
            atomicProperty.getXMLTypeNamespaceURI().equals("typeNamespaceURI"));
        assertTrue(atomicProperty.getModifierString().equals("private"));
        assertTrue(atomicProperty.getEnumeration() == enumeratedValues);
        assertTrue(((String)atomicProperty.getDefaultValue())
            .equals("defaultValue"));
    }

    /**
     * <p>
     *  This tests a (different) valid construction.
     * </p>
     */
    public void testConstructor2() {
        setup();
        atomicProperty = new AtomicProperty("xmlName",
                                            "namespaceURI",
                                            "xmlType",
                                            "typeNamespaceURI");
        assertTrue(atomicProperty != null);
        assertTrue(atomicProperty.getXMLName().equals("xmlName"));
        assertTrue(atomicProperty.getXMLNamespaceURI().equals("namespaceURI"));
        assertTrue(atomicProperty.getXMLType().equals("xmlType"));
        assertTrue(
            atomicProperty.getXMLTypeNamespaceURI().equals("typeNamespaceURI"));
        assertTrue(!atomicProperty.hasDefaultValue());
        assertTrue(atomicProperty.getEnumeration() == null);
    }
    
    /**
     * <p>
     *  This tests a (different) valid construction.
     * </p>
     */
    public void testConstructor3() {
        setup();
        atomicProperty = new AtomicProperty("xmlName", "xmlType");
        assertTrue(atomicProperty != null);
        assertTrue(atomicProperty.getXMLName().equals("xmlName"));
        assertTrue(atomicProperty.getXMLNamespaceURI().equals(""));
        assertTrue(atomicProperty.getXMLType().equals("xmlType"));
        assertTrue(atomicProperty.getXMLTypeNamespaceURI().equals(""));
        assertTrue(!atomicProperty.hasDefaultValue());
        assertTrue(atomicProperty.getEnumeration() == null);
    }
}

